from cms.models.pluginmodel import CMSPlugin
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from django.utils.text import slugify
from django import template

register = template.Library()


class FaqPluginModel(CMSPlugin):
    title = models.CharField(_("Title"), max_length=300)

    class Meta:
        verbose_name = _("Faq Container")

    def __unicode__(self):
        try:
            return f"Faq Container − {self.title} − {self.placeholder.page}"
        except AttributeError:
            return f"Faq Container − {self.title} − " + _("Draft")

    def __str__(self):
        try:
            return f"Faq Container − {self.title} − {self.placeholder.page}"
        except AttributeError:
            return f"Faq Container − {self.title} − " + _("Draft")


class QuestionFaqPluginModel(CMSPlugin):
    question = models.CharField(_("Question"), max_length=300)
    keywords = models.ManyToManyField("djangocms_faq.keyword", verbose_name=_("Keywords"))
    slug = models.SlugField(
        blank=True,
        default='',
        help_text=_("Unique slug for this question. Keep empty to let it be auto-generated."),
        max_length=300,
    )

    @property
    def get_full_url(self):
        try:
            return self.page.get_absolute_url()
        except AttributeError:
            return ""

    def copy_relations(self, oldinstance):
        self.keywords.set(oldinstance.keywords.all())

    class Meta:
        verbose_name = _("Faq Question")

    def __unicode__(self):
        return f"Faq Question − {self.question}"


    def save(self, **kwargs):
        if self.slug == "":
            self.slug = slugify(self.question)
        super().save(**kwargs)

class Keyword(models.Model):
    keyword = models.CharField(_("Keyword"), max_length=100)

    def __unicode__(self):
        return f"{self.keyword}"

    def __str__(self):
        return f"{self.keyword}"


class SearchFaqPluginModel(CMSPlugin):
    name = models.CharField(_("Search name"), max_length=50)
    search_in = models.ManyToManyField("djangocms_faq.faqpluginmodel", verbose_name=_("Search in"), limit_choices_to={'placeholder__page__publisher_is_draft': False})

    def get_faq_published_instances():
        return djangocms_faq.FaqPluginModel.objects.all().filter(placeholder__page__publisher_is_draft=True)

    def copy_relations(self, oldinstance):
        self.search_in.set(oldinstance.search_in.all())

    class Meta:
        verbose_name = _("Faq Search")

    def __unicode__(self):
        return f"Faq Search − {self.name}"



from djangocms_faq.models import QuestionFaqPluginModel

def get_answers(question, search_in):

    if not len(question):
        return

    # answers from question text
    answers_questions = QuestionFaqPluginModel.objects.exclude(
        placeholder__page__publisher_is_draft=True
    ).filter(
        parent__id__in=[id for id in search_in.split()]
    ).filter(
        question__icontains=question
    )

    # answers from keywords list
    answers_keywords = QuestionFaqPluginModel.objects.exclude(
        placeholder__page__publisher_is_draft=True
    ).filter(
        parent__id__in=[id for id in search_in.split()]
    ).filter(
        keywords__keyword__icontains=question
    )

    # all answers
    answers = answers_questions | answers_keywords

    return answers.distinct()



from django.contrib import admin

from .models import Keyword


class DjangocmsFaqKeywordAdmin(admin.ModelAdmin):

    list_display = ("id", "keyword")
    search_field = ("keyword")

    fieldsets = (
        ("Add a keyword", {
            "fields": ("keyword",)
        }),
    )




admin.site.register(
    Keyword, DjangocmsFaqKeywordAdmin
)



from django.conf import settings


ENABLE_API = getattr(settings, "DJANGOCMS_FAQ_ENABLE_API", True)

ANSWER_PLUGINS = getattr(settings, "DJANGOCMS_FAQ_ANSWER_PLUGINS", ["TextPlugin", "FilePlugin", "VideoPlayerPlugin"])

SHOW_KEYWORDS_QUESTION = getattr(settings, "DJANGOCMS_FAQ_SHOW_KEYWORDS_QUESTION", True)
SHOW_KEYWORDS_ANSWER = getattr(settings, "DJANGOCMS_FAQ_SHOW_KEYWORDS_ANSWER", True)



from django.core import serializers
from django.http import HttpResponse
from djangocms_faq.utils import get_answers
from json import dumps

def AnswerView(request):
    if "question" in request.GET:
        question = request.GET['question']

        if "search_in" in request.GET:
            search_in = request.GET["search_in"]
        else:
            search_in = ""

        answers = get_answers(question, search_in)

        jsondata = []

        if answers:
            for answer in answers:
                # look mom I'm creating ugly json!
                jsondata.append({
                    "question": answer.question,
                    "slug": answer.slug,
                    "url": answer.get_full_url,
                    "keywords": [str(keyword) for keyword in answer.keywords.all()]
                })

        return HttpResponse(dumps(jsondata), content_type='application/json')

    return HttpResponse("{}", content_type='application/json')



# Generated by Django 2.2 on 2021-04-06 16:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
    ]

    operations = [
        migrations.CreateModel(
            name='FaqPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='djangocms_faq_faqpluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('title', models.CharField(max_length=300, verbose_name='Titre')),
            ],
            options={
                'verbose_name': 'Faq Container',
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Keyword',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('keyword', models.CharField(max_length=100, verbose_name='Keyword')),
            ],
        ),
        migrations.CreateModel(
            name='SearchFaqPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='djangocms_faq_searchfaqpluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('name', models.CharField(max_length=50, verbose_name='Search name')),
                ('search_in', models.ManyToManyField(limit_choices_to={'placeholder__page__publisher_is_draft': False}, to='djangocms_faq.FaqPluginModel', verbose_name='Search in')),
            ],
            options={
                'verbose_name': 'Faq Search',
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='QuestionFaqPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='djangocms_faq_questionfaqpluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('question', models.CharField(max_length=300, verbose_name='Question')),
                ('slug', models.SlugField(blank=True, default='', help_text='Unique slug for this question. Keep empty to let it be auto-generated.', max_length=300)),
                ('keywords', models.ManyToManyField(to='djangocms_faq.Keyword', verbose_name='Keywords')),
            ],
            options={
                'verbose_name': 'Faq Question',
            },
            bases=('cms.cmsplugin',),
        ),
    ]
